@@ -155,9 +155,10 @@ mpq_class mpq_read(istream& in){
     const string numeric="+-0123456789/.e";
     in >> std::ws;
     string s;
+    char c;
     bool is_float=false;
     while(true){
-        char c = in.peek();
+        c = in.peek();
         size_t pos=numeric.find(c);
         if(pos==string::npos)
             break;
@@ -167,15 +168,28 @@ mpq_class mpq_read(istream& in){
             s+=c;
     }
     
-    if(s=="")
-        throw BadInputException("Error in input file. Most lekely mismatch of amb_space and matrix format.");
+    if(s==""){
+        string t;
+        t+=c;
+        throw BadInputException("Empty number string preceding character "+t+ ". Most likely mismatch of amb_space and matrix format or forgotten keyword.");
+    }
     
-    // cout << "t " << s << " f " << is_float << endl;
+    // cout << "t " << s << " f " << is_float << endl; 
     
-    if(!is_float)
-        return mpq_class(s);
+    if(s[0]=='+')
+        s=s.substr(1); // must suppress + sign for mpq_class
     
-    return dec_fraction_to_mpq(s);
+    try{
+        if(!is_float)
+            return mpq_class(s);
+        else
+            return dec_fraction_to_mpq(s);
+    }
+    catch(const std::exception& e) {
+        cerr << e.what() << endl;
+        cerr << "Illegal number string "+s+" in input, Exiting."  << endl;
+        exit(1); 
+    }
 }
 
 
@@ -197,7 +211,8 @@ bool read_modulus(istream& in, mpq_class& modulus) {
 }
 
 
-void read_symbolic_constraint(istream& in, string& rel, vector<mpq_class>& left, mpq_class& right, mpq_class& modulus, bool forced_hom) {
+/* void read_symbolic_constraint(istream& iFside
+ * n, string& rel, vector<mpq_class>& left, mpq_class& right, mpq_class& modulus, bool forced_hom) {
     
     bool congruence=false;
     bool modulus_read=false;
@@ -298,6 +313,222 @@ void read_symbolic_constraint(istream& in, string& rel, vector<mpq_class>& left,
         continue;
     }
     
+}*/
+
+void read_symbolic_constraint(istream& in, string& rel, vector<mpq_class>& left, mpq_class& right, mpq_class& modulus, bool forced_hom) {
+
+    string constraint;
+    
+    while(true){
+        char c;
+        c=in.get();
+        if(in.fail())
+            throw BadInputException("Symbolic constraint does not end with semicolon");
+        if(c==';')
+            break;
+        constraint+=c;        
+    }
+
+    // remove white space
+    // we must take care that the removal of white space does not
+    // shadow syntax errors
+    string without_spaces;
+    bool digit_then_spaces=false;
+    bool has_content=false;
+    for(size_t j=0;j<constraint.size();++j){
+        char test=constraint[j];
+        if(!isspace(test))
+            has_content=true;
+        if(isspace(test))
+            continue;
+        if(test=='.'){
+            if(j==constraint.size()-1 || isspace(constraint[j+1]))
+                throw BadInputException("Incomplete number");
+        }
+        if(test=='e'){
+            if(j==constraint.size()-1 || isspace(constraint[j+1]))
+                throw BadInputException("Incomplete number");
+            if(j<=constraint.size()-3 && (constraint[j+1]=='+' || constraint[j+1]=='-')
+                && isspace(constraint[j+2]))
+                    throw BadInputException("Incomplete number");
+        }
+        if(!isdigit(test))
+            digit_then_spaces=false;
+        else{
+            if(digit_then_spaces)
+                throw BadInputException("Incomplete number");
+            // cout << "jjjj " << j << " |" << constraint[j+1] << "|" << endl;
+            if(j<constraint.size()-1 && isspace(constraint[j+1])){
+                digit_then_spaces=true;
+                // cout << "Drin" << endl;
+            }
+        }            
+        without_spaces+=test;               
+    }
+    if(!has_content)
+        throw BadInputException("Empty symbolic constraint");
+    
+    // split into terms
+    // we separate by + and -
+    // except: first on lhs or rhs, between ( and ) and following e.
+    bool first_sign=true;
+    bool in_brackets=false;
+    bool relation_read=false;
+    size_t RHS_start=0;
+    vector<string> terms;
+    string current_term;
+    for(size_t j=0;j<without_spaces.size();++j){
+        char test=without_spaces[j];
+        if(test=='(')
+            in_brackets=true;
+        if(test==')'){
+            if(!in_brackets)
+                throw BadInputException("Closing bracket without opening bracket");
+            in_brackets=false;
+        }
+        if(test=='+' || test=='-'){
+            if(!first_sign && !in_brackets){
+                terms.push_back(current_term);
+                current_term.clear();
+            }
+        }
+        first_sign=false;
+        
+        if(test=='e'){
+            current_term+=test;
+            if(j==without_spaces.size()-1)
+                throw BadInputException("Incomplete number");
+            if(without_spaces[j+1]=='+' || without_spaces[j+1]=='-'){
+                current_term+=without_spaces[j+1];
+                j++;
+            }
+            continue;
+        }
+        
+        if(test=='=' || test=='<' || test=='>' || test=='~'){
+            terms.push_back(current_term);
+            current_term.clear();
+            rel+=test;
+            RHS_start=terms.size();
+            if(relation_read)
+                throw BadInputException("Double relation in constraint");
+            relation_read=true;
+            if(j==without_spaces.size()-1)
+                throw BadInputException("Relation last character in constraint");
+            if(without_spaces[j+1]=='='){
+                rel+=without_spaces[j+1];
+                j++;
+            }
+            first_sign=true;
+            continue;
+        }
+        
+        current_term+=test;
+    }
+    terms.push_back(current_term);
+    if(!relation_read)
+        throw BadInputException("No relation in constraint");
+    
+    // for(size_t i=0;i<terms.size();++i)
+     //   cout << i << ": " << terms[i] << "| " << terms[i].size() << endl;
+    
+    //now we split off the modulus if necessary
+    if(rel=="~"){
+        string last_term=terms.back();
+        size_t last_bracket_at=0;
+        bool has_bracket=false;
+        for(size_t i=0;i<last_term.size();++i){
+            if(last_term[i]=='('){
+                last_bracket_at=i;
+                has_bracket=true;                
+            }      
+        }
+        if(!has_bracket || last_term.back()!=')')
+            throw BadInputException("Error in modulus of congruence");
+        string modulus_string=last_term.substr(last_bracket_at+1,last_term.size()-last_bracket_at-2);
+        terms.back()=last_term.substr(0,last_bracket_at);
+        if(terms.back()=="")
+            terms.pop_back();
+        modulus=mpq_class(modulus_string);
+        modulus.canonicalize();
+        // cout << "mod " << modulus << endl;
+        if(modulus <=0 || modulus.get_den()!=1)
+            throw BadInputException("Error in modulus of congruence");        
+    }
+    
+    // for(size_t i=0;i<terms.size();++i)
+    //     cout << i << ": " << terms[i] << "| " << terms[i].size() << endl;
+    
+    // now we must process the terns
+    
+    right=0;
+    mpq_class side=1;
+    
+    for(size_t i=0;i<terms.size();++i){
+        
+        if(i==RHS_start)
+            side=-1;
+        
+        string& this_term =terms[i];
+        if(this_term=="")
+            throw BadInputException("Empty term in symbolic constraint");
+        if(this_term=="+" || this_term=="-")
+            throw BadInputException("Double sign or incomplete number");
+        size_t coeff_length=0;
+        for(size_t j=0;j<this_term.size();++j){
+            if(this_term[j]!='x')
+                coeff_length++;
+            else
+                break;
+        }
+        string coeff_string=this_term.substr(0,coeff_length);
+        string comp_string=this_term.substr(coeff_length,this_term.size()-coeff_length);
+        mpq_class coeff=0;
+        if(coeff_length==0 || (coeff_length==1 && coeff_string[0]=='+'))
+            coeff=1;
+        if(coeff_length==1 && coeff_string[0]=='-')
+            coeff=-11;
+        if(coeff==0){
+            // cout << i << " coeff string: " << coeff_string << endl;
+            const string numeric="+-0123456789/.e";
+            for(size_t j=0;j<coeff_string.size();++j){
+                size_t pos=numeric.find(coeff_string[j]);
+                if(pos==string::npos)
+                    throw BadInputException("Illegal character in number");
+            }
+            
+            stringstream  for_coeff;
+            for_coeff << coeff_string;
+            coeff=mpq_read(for_coeff);            
+        }
+        if(comp_string!=""){
+            bool bracket_read=false;
+            string expo_string;
+            for(size_t j=0;j<comp_string.size();++j){
+                if(comp_string[j]==']')
+                    break;
+                if(comp_string[j]=='['){
+                    bracket_read=true;
+                    continue;
+                }
+                if(bracket_read)
+                    expo_string+=comp_string[j];
+            }
+            if(expo_string.size()!=comp_string.size()-3)
+                throw BadInputException("Error in naming variable in symbolic constraint");
+            
+            long index=stol(expo_string);
+            if(index <1 || index > (long) left.size())
+                throw BadInputException("Index " + expo_string +" in symbolic constraint out of bounds");
+            index--;
+            left[index]+=side*coeff;
+        }
+        else{ // absolute term
+            right-=side*coeff;
+        }
+        
+        // cout << "constraint " << left << rel << " " << right << endl;
+    }
 }
 
 
@@ -333,7 +564,7 @@ void read_constraints(istream& in, long dim, map <Type::InputType, vector< vecto
     for(long i=0;i< nr_constraints; ++i) {
         
         vector<mpq_class> left(dim-hom_correction);
-        string rel, modulus_str;
+        string rel;
         mpq_class right, modulus=0;
         
         if(symbolic){
@@ -518,7 +749,7 @@ map <Type::InputType, vector< vector<mpq_class> > > readNormalizInput (istream&
         }
         else{            
             in >> dim;
-            if (!in.good() || dim <= 0) {
+            if (!in.good() || dim < 0) {
                 throw BadInputException("Bad amb_space value!");
             }
             dim_known=true;
@@ -556,6 +787,10 @@ map <Type::InputType, vector< vector<mpq_class> > > readNormalizInput (istream&
                     options.activateNoExtRaysOutput();
                     continue;
                 }
+                if (type_string == "NoSuppHypsOutput") {
+                    options.activateNoSuppHypsOutput();
+                    continue;
+                }
                 if (type_string == "total_degree") {
                     if(!dim_known){
                         throw BadInputException("Ambient space must be known for "+type_string+"!");
@@ -807,9 +1042,8 @@ map <Type::InputType, vector< vector<mpq_class> > > readNormalizInput (istream&
                 break;
             in >> nr_columns;
             if((nr_rows <0) || (nr_columns < 0)){
-                throw BadInputException("Error while reading a "
-                        + toString(nr_rows) + "x" + toString(nr_columns)
-                        + " matrix !");
+                throw BadInputException("Error while reading matrix format "
+                        + toString(nr_rows) + "x" + toString(nr_columns));
             }
             vector< vector<mpq_class> > M(nr_rows,vector<mpq_class>(nr_columns));
             for(i=0; i<nr_rows; i++){
@@ -822,7 +1056,7 @@ map <Type::InputType, vector< vector<mpq_class> > > readNormalizInput (istream&
             in >> type_string;
 
             if ( in.fail() ) {
-                throw BadInputException("Error while reading a " 
+                throw BadInputException("Error while reading type string of " 
                         + toString(nr_rows) + "x" + toString(nr_columns)
                         + " matrix!");
             }
