@@ -29,6 +29,8 @@
 #include <math.h>
 #include <iomanip>
 
+#include <boost/dynamic_bitset.hpp>
+
 #include "libnormaliz/matrix.h"
 #include "libnormaliz/cone.h"
 #include "libnormaliz/vector_operations.h"
@@ -211,10 +213,10 @@ void Matrix<Integer>::pretty_print(ostream& out, bool with_row_nr) const{
     size_t max_index_length = decimal_length(nr);
     for (i = 0; i < nr; i++) {
         if (with_row_nr) {
-            out << std::setw(max_index_length+1) << i<< ": ";
+            out << std::setw(max_index_length+1) << std::setprecision(6) << i<< ": ";
         }
         for (j = 0; j < nc; j++) {
-            out << std::setw(max_length[j]+1) << elem[i][j];
+            out << std::setw(max_length[j]+1)  << std::setprecision(6) << elem[i][j];
         }
         out<<endl;
     }
@@ -363,6 +365,28 @@ Matrix<Integer> Matrix<Integer>::submatrix(const vector<bool>& rows) const{
     return M;
 }
 
+
+/*//---------------------------------------------------------------------------
+
+template<typename Integer>
+Matrix<Integer> Matrix<Integer>::submatrix(const boost::dynamic_bitset<>& rows) const{
+    assert(rows.size() == nr);
+    size_t size=0;
+    for (size_t i = 0; i <rows.size(); i++) {
+        if (rows[i]) {
+            size++;
+        }
+    }
+    Matrix<Integer> M(size, nc);
+    size_t j = 0;
+    for (size_t i = 0; i < nr; i++) {
+        if (rows[i]) {
+            M.elem[j++] = elem[i];
+        }
+    }
+    return M;
+}*/
+
 //---------------------------------------------------------------------------
 
 template<typename Integer>
@@ -2150,7 +2174,7 @@ size_t Matrix<nmz_float>::row_echelon(){
 //---------------------------------------------------------------------------
 
 template<typename Integer>
-Matrix<Integer> Matrix<Integer>::kernel () const{
+Matrix<Integer> Matrix<Integer>::kernel (bool use_LLL) const{
 // computes a ZZ-basis of the solutions of (*this)x=0
 // the basis is formed by the rOWS of the returned matrix
 
@@ -2173,7 +2197,10 @@ Matrix<Integer> Matrix<Integer>::kernel () const{
     Matrix<Integer> Help =Transf.transpose();
     for (size_t i = rank; i < dim; i++) 
             ker_basis[i-rank]=Help[i];
-    return ker_basis.LLL();
+    if(use_LLL)
+        return ker_basis.LLL();
+    else
+        return ker_basis;
     //ker_basis.row_echelon_reduce();
     //return(ker_basis);
 }
@@ -2363,7 +2390,7 @@ void mpz_submatrix_trans(Matrix<mpz_class>& sub, const Matrix<Integer>& mother,
 template<typename Integer>
 vector<key_t> Matrix<Integer>::perm_sort_by_degree(const vector<key_t>& key, const vector<Integer>& grading, bool computed) const{
 
-            list<vector<Integer>> rowList;
+            list<vector<Integer> > rowList;
             vector<Integer> v;
 
             v.resize(nc+2);
@@ -2798,7 +2825,7 @@ vector<Integer> Matrix<Integer>::optimal_subdivision_point_inner() const{
 // The orthogonal matrix is B
 // Coefficients in M
 template<typename Integer>
-void Matrix<Integer>::GramSchmidt(Matrix<double>& B, Matrix<double>& M, int from, int to){
+void Matrix<Integer>::GramSchmidt(Matrix<nmz_float>& B, Matrix<nmz_float>& M, int from, int to){
 
     // from=0;
     // to= (int) nr_of_rows();
@@ -2808,9 +2835,9 @@ void Matrix<Integer>::GramSchmidt(Matrix<double>& B, Matrix<double>& M, int from
         convert(B[i],elem[i]);
         // cout << B[i];
         for(int j=0;j<i;++j){
-            double sp=0;
+            nmz_float sp=0;
             for(size_t k=0;k<dim;++k){
-                double fact;
+                nmz_float fact;
                 convert(fact,elem[i][k]);
                 sp+=fact*B[j][k];
             }
@@ -2839,8 +2866,8 @@ Matrix<Integer> Matrix<Integer>::LLL_red(Matrix<Integer>& T, Matrix<Integer>& Ti
     if(n<=1)
         return Lred;
     
-    Matrix<double> G(n,dim);
-    Matrix<double> M(n,n);
+    Matrix<nmz_float> G(n,dim);
+    Matrix<nmz_float> M(n,n);
     
     Lred.GramSchmidt(G,M,0,2);
     
@@ -2863,8 +2890,8 @@ Matrix<Integer> Matrix<Integer>::LLL_red(Matrix<Integer>& T, Matrix<Integer>& Ti
             Lred.GramSchmidt(G,M,0,2);
             continue;
         }
-        double t1=v_scalar_product(G[i-1],G[i-1]);
-        double t2=v_scalar_product(G[i],G[i]);
+        nmz_float t1=v_scalar_product(G[i-1],G[i-1]);
+        nmz_float t2=v_scalar_product(G[i],G[i]);
         if(t1> 2*t2){
             std::swap(Lred[i],Lred[i-1]);
             std::swap(T[i],T[i-1]);
@@ -2886,77 +2913,12 @@ Matrix<Integer> Matrix<Integer>::LLL_red(Matrix<Integer>& T, Matrix<Integer>& Ti
     return Lred;
 }*/
 
-template<typename Integer>
-Matrix<Integer> Matrix<Integer>::LLL_red(Matrix<Integer>& T, Matrix<Integer>& Tinv) const{
-// returns Lred =LLL_reduced(L) (sublattice generated by the rows!)
-// Lred=T*this, Tinv=inverse(T)
-// Original version with c = 0.9
-    
-    T=Tinv=Matrix<Integer>(nr);
-    
-    Matrix<Integer> Lred=*this;
-    size_t dim=nr_of_columns();
-    int n=nr_of_rows();
-    // pretty_print(cout);
-    assert((int) rank()==n);
-    if(n<=1)
-        return Lred;
-    
-    Matrix<double> G(n,dim);
-    Matrix<double> M(n,n);
-    
-    Lred.GramSchmidt(G,M,0,2);
-    
-    int i=1;
-    while(true){
-        
-        for(int j=i-1;j>=0;--j){
-            Integer fact;
-            /* cout << "MMMMM " << i << " " << j << " " << M[i][j] << endl;
-            cout << i << "---" << G[i];
-            cout << j << "---" << G[j];*/
-            convert(fact,round(M[i][j]));
-            // cout << fact << " " << M[i][j] << endl;
-            if(fact!=0){
-                v_el_trans<Integer>(Lred[j],Lred[i],-fact,0);
-                v_el_trans<Integer>(T[j],T[i],-fact,0);
-                v_el_trans<Integer>(Tinv[i],Tinv[j],fact,0);
-                Lred.GramSchmidt(G,M,i,i+1); 
-            }
-        }
-        if(i==0){
-            i=1;
-            Lred.GramSchmidt(G,M,0,2);
-            continue;
-        }
-        double t1=v_scalar_product(G[i-1],G[i-1]);
-        double t2=v_scalar_product(G[i],G[i]);
-        double fact=0.9-M[i][i-1]*M[i][i-1];
-        if(t2<fact*t1){
-            std::swap(Lred[i],Lred[i-1]);
-            std::swap(T[i],T[i-1]);
-            std::swap(Tinv[i],Tinv[i-1]);
-            Lred.GramSchmidt(G,M,i-1,i); // i-1,i+1);
-            // cout << i-1 << "---" << G[i-1];
-            i--;
-        }
-        else{
-            i++;
-            if(i>=n)
-                break;
-            Lred.GramSchmidt(G,M,i,i+1);
-        }
-    }
-    
-    Tinv=Tinv.transpose();
-    
-    return Lred;
-}
+
 
 template<typename Integer>
 Matrix<Integer> Matrix<Integer>::LLL() const{
     Matrix<Integer> Dummy1,Dummy2;
-    return LLL_red(Dummy1,Dummy2);
+    return LLL_red(*this,Dummy1,Dummy2);
 }
 
 template<typename Integer>
@@ -2965,19 +2927,6 @@ Matrix<Integer> Matrix<Integer>::LLL_transpose() const{
     return transpose().LLL().transpose();
 }
 
-template<typename Integer>
-Matrix<Integer> Matrix<Integer>::LLL_red_transpose(Matrix<Integer>& T, Matrix<Integer>& Tinv) const{
-// column version -- needed for coordinate transformations in ambient lattice
-// returns Lred=this*T, Tinv=inverse(T)    
-    
-    Matrix<Integer> this_trans=transpose();
-    Matrix<Integer> red_trans,T_trans, Tinv_trans;
-    red_trans=this_trans.LLL_red(T_trans,Tinv_trans);
-    T=T_trans.transpose();
-    Tinv=Tinv_trans.transpose();
-    return red_trans.transpose();
-}
-
 #ifndef NMZ_MIC_OFFLOAD  //offload with long is not supported
 template Matrix<long>  readMatrix(const string project);
 #endif // NMZ_MIC_OFFLOAD
@@ -2993,7 +2942,8 @@ template class Matrix<nmz_float>;
 // result returned in is_max_subset -- must be initialized outside
 // only set to false in this routine
 // if a set occurs more than once, only the last instance is recognized as maximal
-void maximal_subsets(const vector<vector<bool> >& ind, vector<bool>& is_max_subset) {
+template<typename IncidenceVector>
+void maximal_subsets(const vector<IncidenceVector>& ind, vector<bool>& is_max_subset) {
 
     if(ind.size()==0)
         return;
@@ -3029,6 +2979,8 @@ void maximal_subsets(const vector<vector<bool> >& ind, vector<bool>& is_max_subs
         }
     }
 }
+template void  maximal_subsets(const vector<vector<bool> >&, vector<bool>& );
+template void  maximal_subsets(const vector<boost::dynamic_bitset<> >&, vector<bool>& );
 
 
 }  // namespace
