@@ -392,6 +392,19 @@ try{
   IsRational(RFrat,remainingFactor); // from RingQQ to BigRat
   I*=RFrat;
   
+  // We integrate over the polytope P which is the intersection of the cone
+  // with the hyperplane at degree 1. Our transformation formula applied
+  // is only correct ifassumes that P hathe same lattice volume as
+  // the convex hull of P and 0. Lattice volume comes from the effective lattice. 
+  // Therefore we need a correction factor if the restriction of the absolute
+  // grading to the effective lattice is (grading on eff latt)/g with g>1.
+  // this amounts to multiplying the integral by g.
+
+  vector<Integer> test_grading=C.getSublattice().to_sublattice_dual_no_div(C.getGrading());
+  Integer corr_factor=v_gcd(test_grading);  
+  mpz_class corr_mpz=convertTo<mpz_class>(corr_factor);
+  I*=BigInt(corr_mpz.get_mpz_t());  
+  
   string result="Integral";
   if(do_virt_mult)
     result="Virtual multiplicity";
@@ -402,8 +415,14 @@ try{
     VM*=factorial(deg(F)+rank-1);
     C.getIntData().setVirtualMultiplicity(mpq(VM));
   }
-  else
+  else{
+    BigRat I_fact=I*factorial(rank-1);  
+    mpq_class Int_bridge=mpq(I_fact);
+    nmz_float EuclInt=mpq_to_nmz_float(Int_bridge);
+    EuclInt*=C.euclidean_corr_factor();  
     C.getIntData().setIntegral(mpq(I));
+    C.getIntData().setEuclideanIntegral(EuclInt);
+  }
 
    if(verbose_INT){
     verboseOutput() << "********************************************" << endl;
