@@ -206,9 +206,10 @@ Integer SimplexEvaluator<Integer>::start_evaluation(SHORTSIMPLEX<Integer>& s, Co
     //degrees of the generators according to the Grading of C
     if(C.isComputed(ConeProperty::Grading))
         for (i=0; i<dim; i++){
-            if(C.do_h_vector)
-                gen_degrees_long[i] = convertTo<long>(C.gen_degrees[key[i]]);
-            gen_degrees[i]=C.gen_degrees[key[i]];
+            if(!do_only_multiplicity || C.inhomogeneous || using_GMP<Integer>())
+                gen_degrees[i]=C.gen_degrees[key[i]];
+            if(C.do_h_vector || !using_GMP<Integer>())
+                gen_degrees_long[i] = C.gen_degrees_long[key[i]];
         }
             
     nr_level0_gens=0;
@@ -618,9 +619,17 @@ void SimplexEvaluator<Integer>::reduce_against_global(Collector<Integer>& Coll)
     
     Full_Cone<Integer>& C = *C_ptr;
     bool inserted;
-    typename list< vector<Integer> >::iterator jj = Hilbert_Basis.begin();
+    auto jj = Hilbert_Basis.begin();
     for(;jj != Hilbert_Basis.end();++jj) {
         jj->pop_back(); //remove the norm entry at the end
+    
+        if(C.inhomogeneous && C.hilbert_basis_rec_cone_known){ // skip elements of the precomputed Hilbert basis
+            Integer level_Int=0;
+            for(size_t i=0;i<dim;i++)
+                level_Int+=(*jj)[i]*gen_levels[i];
+            if(level_Int==0)
+                continue;
+        }
         if (!isDuplicate(*jj)) { //skip the element
             
             // cout << "Vor " << *jj;
@@ -1149,16 +1158,26 @@ void SimplexEvaluator<Integer>::addMult(Integer multiplicity, Collector<Integer>
     if (C_ptr->deg1_triangulation) {
         Coll.mult_sum += convertTo<mpz_class>(multiplicity);
     } else {
-        mpz_class deg_prod=convertTo<mpz_class>(gen_degrees[0]);
-        for (size_t i=1; i<dim; i++) {
-            deg_prod *= convertTo<mpz_class>(gen_degrees[i]);
+        if(using_GMP<Integer>()){
+            mpz_class deg_prod=convertTo<mpz_class>(gen_degrees[0]);
+            for (size_t i=1; i<dim; i++) {
+                deg_prod *= convertTo<mpz_class>(gen_degrees[i]);
+            }
+            mpq_class mult = convertTo<mpz_class>(multiplicity);
+            mult /= deg_prod;
+            Coll.mult_sum += mult;
+        }
+        else{
+            mpz_class deg_prod=gen_degrees_long[0];
+            for (size_t i=1; i<dim; i++) {
+                deg_prod *= gen_degrees_long[i];
+            }
+            mpq_class mult = convertTo<mpz_class>(multiplicity);
+            mult /= deg_prod;
+            Coll.mult_sum += mult;
         }
-        mpq_class mult = convertTo<mpz_class>(multiplicity);
-        mult /= deg_prod;
-        Coll.mult_sum += mult;
     }  
 }
-
 //---------------------------------------------------------------------------
 
 template<typename Integer>
