@@ -30,6 +30,10 @@
 #include <csignal>
 using namespace std;
 
+#ifdef NMZ_GPERF
+#include <gperftools/profiler.h>
+#endif
+
 #include "normaliz.h"
 #include "libnormaliz/integer.h"
 #include "libnormaliz/libnormaliz.h"
@@ -53,7 +57,7 @@ void printHeader() {
                                                  << "             \\....|"<<endl;
     cout << "                                                      \\...|"<<endl;
     cout << "     (C) The Normaliz Team, University of Osnabrueck   \\..|"<<endl;
-    cout << "                    January  2018                       \\.|"<<endl;
+    cout << "                    July  2018                          \\.|"<<endl;
     cout << "                                                         \\|"<<endl;
     bool with_optional_packages=false;
     string optional_packages;
@@ -71,7 +75,7 @@ void printHeader() {
 #endif
     if(with_optional_packages){    
         cout << "------------------------------------------------------------" << endl;
-        cout << "with paackage(s)" << optional_packages << endl; 
+        cout << "with package(s)" << optional_packages << endl; 
     }
 }
 
@@ -84,6 +88,7 @@ void printHelp(char* command) {
     cout << "  -t\tcompute triangulation"<<endl;
     cout << "  -v\tcompute multiplicity"<<endl;
     cout << "  -v\tcompute volume"<<endl;
+    cout << "  -F\tcompute volume by decent in the face lattice"<<endl;
     cout << "  -n\tcompute Hilbert basis and multiplicity (needs full triangulation)"<<endl;
     cout << "  -N\tcompute Hilbert basis (with partial triangulation)"<<endl;
     cout << "  -w\tcheck for integrally closed and compute witness if not"<<endl;
@@ -137,7 +142,7 @@ void printHelp(char* command) {
 }
 
 void printCopying() {
-    cout<<"Copyright (C) 2007-2017  The Normaliz Team, University of Osnabrueck."<<endl
+    cout<<"Copyright (C) 2007-2018  The Normaliz Team, University of Osnabrueck."<<endl
         <<"This program comes with ABSOLUTELY NO WARRANTY; This is free software,"<<endl
         <<"and you are welcome to redistribute it under certain conditions;"<<endl
         <<"See COPYING for details."<<endl;
@@ -155,6 +160,10 @@ int process_data(OptionsHandler& options, const string& command_line,const strin
 
 int main(int argc, char* argv[])
 {
+
+#ifdef NMZ_GPERF
+    ProfilerStart("normaliz.prof");
+#endif
     
     // signal handler for interrupt
     signal(SIGINT, &interrupt_signal_handler);
@@ -184,7 +193,11 @@ int main(int argc, char* argv[])
     
     if(nmz_interrupted)
         exit(10);
-
+    
+#ifdef NMZ_GPERF
+    ProfilerStop();
+#endif
+    
     exit(0);
 }
 
@@ -265,6 +278,10 @@ int process_data(OptionsHandler& options, const string& command_line,const strin
     try {
 #endif
 
+    if(options.getProjectName()==""){
+        cerr << "ERROR: No project name set!" << endl;
+        exit(1);
+    }
 
     string name_in=options.getProjectName()+".in";
     const char* file_in=name_in.c_str();
