@@ -47,6 +47,7 @@ OptionsHandler::OptionsHandler() {
     ignoreInFileOpt = false;
     nr_threads = 0;
     no_ext_rays_output=false;
+    no_supp_hyps_output=false;
 }
 
 
@@ -145,6 +146,9 @@ bool OptionsHandler::handle_options(vector<string>& LongOptions, string& ShortOp
                 to_compute.set(ConeProperty::Triangulation);
                 // to_compute.set(ConeProperty::Multiplicity);
                 break;
+            case 'F':
+                to_compute.set(ConeProperty::Descent);
+                break;
             case 's':
                 to_compute.set(ConeProperty::SupportHyperplanes);
                 break;
@@ -311,6 +315,10 @@ bool OptionsHandler::handle_options(vector<string>& LongOptions, string& ShortOp
             no_ext_rays_output=true;
             continue;
         }
+        if(LongOptions[i]=="NoSuppHypsOutput"){
+            no_supp_hyps_output=true;
+            continue;
+        }
         if(LongOptions[i]=="ignore"){
             ignoreInFileOpt=true;
             continue;
@@ -350,6 +358,8 @@ template<typename Integer>
 void OptionsHandler::applyOutputOptions(Output<Integer>& Out) {
     if(no_ext_rays_output)
         Out.set_no_ext_rays_output();
+    if(no_supp_hyps_output)
+        Out.set_no_supp_hyps_output();
     if(write_all_files) {
         Out.set_write_all_files();
     } else if (write_extra_files) {
@@ -420,10 +430,7 @@ void OptionsHandler::applyOutputOptions(Output<Integer>& Out) {
 }
 
 bool OptionsHandler::activateDefaultMode() {
-    if (to_compute.goals().none() && !to_compute.test(ConeProperty::DualMode) 
-                                  && !to_compute.test(ConeProperty::Approximate)
-                                  && !to_compute.test(ConeProperty::ProjectionFloat)
-                                  && !to_compute.test(ConeProperty::Projection)  ) {
+    if (to_compute.goals().none() && !to_compute.test(ConeProperty::DualMode) ){
         to_compute.set(ConeProperty::DefaultMode);
         return true;
     }
@@ -447,4 +454,4 @@ string pureName(const string& fullName){
     // exit(1);
     return(fullName.substr(found,length));  	
 
-}
+}
\ No newline at end of file
