@@ -59,6 +59,7 @@ Output<Integer>::Output(){
     msp=false;
     lattice_ideal_input = false;
     no_ext_rays_output=false;
+    no_supp_hyps_output=false;
 }
 
 //---------------------------------------------------------------------------
@@ -70,6 +71,13 @@ void Output<Integer>::set_lattice_ideal_input(bool value){
 
 //---------------------------------------------------------------------------
 
+template<typename Integer>
+void Output<Integer>::set_no_supp_hyps_output(){
+    no_supp_hyps_output=true;
+}
+
+//---------------------------------------------------------------------------
+
 template<typename Integer>
 void Output<Integer>::set_no_ext_rays_output(){
     no_ext_rays_output=true;
@@ -109,14 +117,23 @@ void Output<Integer>::setCone(Cone<Integer> & C) {
     this->Result = &C;
     dim = Result->getEmbeddingDim();
     homogeneous = !Result->isInhomogeneous();
+    HilbertOrEhrhart="Hilbert ";
     if (homogeneous) {
         of_cone       = "";
         of_monoid     = "";
         of_polyhedron = "";
+        string absolute;
+        module_generators_name = " lattice points in polytope (Hilbert basis elements of degree 1)";
     } else {
         of_cone       = " of recession cone";
         of_monoid     = " of recession monoid";
         of_polyhedron = " of polyhedron (homogenized)";
+        if(Result->isComputed(ConeProperty::ModuleGenerators) && Result->getRecessionRank()==0)
+            module_generators_name=" lattice points in polytope (module generators)";
+        else
+            module_generators_name=" module generators";
+        if(Result->isComputed(ConeProperty::EhrhartSeries))
+            HilbertOrEhrhart="Ehrhart ";
     }
 }
 
@@ -344,8 +361,8 @@ void Output<Integer>::write_tri() const{
         out << Tri.size() << endl;
         size_t nr_extra_entries=1;
         if (Result->isComputed(ConeProperty::ConeDecomposition))
-            nr_extra_entries+=Result->getSublattice().getRank();
-        out << Result->getSublattice().getRank()+nr_extra_entries << endl; //works also for empty list
+            nr_extra_entries+=Result->getSublattice().getRank()-Result->getDimMaximalSubspace();
+        out << Result->getSublattice().getRank()-Result->getDimMaximalSubspace()+nr_extra_entries << endl; //works also for empty list
 
         for(; tit != Tri.end(); ++tit) {
             for (size_t i=0; i<tit->first.size(); i++) {
@@ -709,7 +726,10 @@ void Output<Integer>::writeWeightedEhrhartSeries(ofstream& out) const{
         
     if(Result->isComputed(ConeProperty::VirtualMultiplicity)){
         out << endl << "Virtual multiplicity: ";
-        out << Result->getIntData().getVirtualMultiplicity() << endl << endl;
+        out << Result->getIntData().getVirtualMultiplicity() << endl;
+        if(Result->getIntData().getVirtualMultiplicity().get_den()!=1)
+                out << "Virtual multiplicity (float): "<< std::setprecision(12) << mpq_to_nmz_float(Result->getIntData().getVirtualMultiplicity()) << endl;
+        out << endl;
     }
 }
 
@@ -764,14 +784,14 @@ void Output<Integer>::write_files() const {
             out << nr_orig_gens <<" original generators of the toric ring"<<endl;
         }
         if (Result->isComputed(ConeProperty::ModuleGenerators)) {
-            out << Result->getNrModuleGenerators() <<" module generators" << endl;
+            out << Result->getNrModuleGenerators() << module_generators_name << endl;
         }
         if (Result->isComputed(ConeProperty::HilbertBasis)) {
             out << Result->getNrHilbertBasis() <<" Hilbert basis elements"
                 << of_monoid << endl;
         }
         if (homogeneous && Result->isComputed(ConeProperty::Deg1Elements)) {
-            out << Result->getNrDeg1Elements() <<" Hilbert basis elements of degree 1"<<endl;
+            out << Result->getNrDeg1Elements() << module_generators_name << endl;
         }
         if (Result->isComputed(ConeProperty::IsReesPrimary)
             && Result->isComputed(ConeProperty::HilbertBasis)) {
@@ -813,8 +833,12 @@ void Output<Integer>::write_files() const {
             if (Result->isComputed(ConeProperty::AffineDim))
                 out << "affine dimension of the polyhedron = "
                     << Result->getAffineDim() << is_maximal(Result->getAffineDim(),dim-1) << endl;
-            if (Result->isComputed(ConeProperty::RecessionRank))
-                out << "rank of recession monoid = "  << Result->getRecessionRank() << endl;
+            if (Result->isComputed(ConeProperty::RecessionRank)){
+                out << "rank of recession monoid = "  << Result->getRecessionRank();
+                if(Result->getRecessionRank()==0)
+                    out << " (polyhedron is polytope)";                
+                out << endl;
+            }
         }
         
         if(Result->isComputed(ConeProperty::OriginalMonoidGenerators)){
@@ -879,7 +903,7 @@ void Output<Integer>::write_files() const {
             }
         }
         else if (Result->isComputed(ConeProperty::IsDeg1ExtremeRays)) {
-            if ( !Result->isDeg1ExtremeRays() ) {
+            if ( !Result->isDeg1ExtremeRays()) {
                 out << "No implicit grading found" << endl;
             }
         }
@@ -898,10 +922,14 @@ void Output<Integer>::write_files() const {
         }
         if ( Result->isComputed(ConeProperty::Multiplicity) ) {
             out << "multiplicity = "<< Result->getMultiplicity() << endl;
+            if(Result->getMultiplicity().get_den()!=1)
+                out << "multiplicity (float) = "<< std::setprecision(12) << mpq_to_nmz_float(Result->getMultiplicity()) << endl;
         }
         if ( Result->isComputed(ConeProperty::Volume) && Result->isComputed(ConeProperty::Sublattice)) {
-            out << "volume (normalized) = "<< Result->getVolume() << endl;            
-            out << "volume (Euclidean) = "<< Result->getEuclideanVolume() << endl;
+            out << "volume (normalized) = "<< std::setprecision(12) << Result->getVolume() << endl;
+            if(Result->getVolume().get_den()!=1)
+                out << "volume (normalized, float) = "<< mpq_to_nmz_float(Result->getVolume()) << endl;
+            out << "volume (Euclidean) = "<< std::setprecision(12) << Result->getEuclideanVolume() << endl;
         }
         if ( Result->isComputed(ConeProperty::ModuleRank) || Result->isComputed(ConeProperty::Multiplicity) 
                     || Result->isComputed(ConeProperty::Volume)) {
@@ -915,11 +943,11 @@ void Output<Integer>::write_files() const {
             if ( Result->isComputed(ConeProperty::HSOP) ){
                     HS_Denom=HS.getHSOPDenom();
                     HS_Num=HS.getHSOPNum();
-                    out << "Hilbert series (HSOP):" << endl << HS_Num;
+                    out << HilbertOrEhrhart+"series (HSOP):" << endl << HS_Num;
             } else {
                     HS_Denom=HS.getDenom();
                     HS_Num=HS.getNum();
-                    out << "Hilbert series:" << endl << HS_Num;
+                    out << HilbertOrEhrhart+"series:" << endl << HS_Num;
             }
             long nr_factors = 0;
             for (map<long, long>::iterator it = HS_Denom.begin(); it!=HS_Denom.end(); ++it) {
@@ -932,14 +960,14 @@ void Output<Integer>::write_files() const {
                 out << "shift = " << HS.getShift() << endl << endl;
             }
             
-            out << "degree of Hilbert Series as rational function = "
+            out << "degree of " +HilbertOrEhrhart+ "Series as rational function = "
                 << HS.getDegreeAsRationalFunction() << endl << endl;
             if(v_is_symmetric(HS_Num)){
-                out << "The numerator of the Hilbert Series is symmetric." << endl << endl;
+                out << "The numerator of the "+HilbertOrEhrhart+"series is symmetric." << endl << endl;
             }
             if(HS.get_expansion_degree()>-1){
                 vector<mpz_class> expansion=HS.getExpansion();
-                out << "Expansion of Hilbert series" << endl;
+                out << "Expansion of "+HilbertOrEhrhart + "series" << endl;
                 for(size_t i=0;i<expansion.size();++i)
                     out << i+HS.getShift()  << ": " << expansion[i] << endl;
                 out << endl;
@@ -947,14 +975,14 @@ void Output<Integer>::write_files() const {
             long period = HS.getPeriod();
             if (period == 1 && (HS_Denom.size() == 0
                                 || HS_Denom.begin()->first== (long) HS_Denom.size())) {
-                out << "Hilbert polynomial:" << endl;
+                out << HilbertOrEhrhart+ "polynomial:" << endl;
                 out << HS.getHilbertQuasiPolynomial()[0];
                 out << "with common denominator = ";
                 out << HS.getHilbertQuasiPolynomialDenom();
                 out << endl<< endl;
             } else {
                 // output cyclonomic representation
-                out << "Hilbert series with cyclotomic denominator:" << endl;
+                out << HilbertOrEhrhart << "series with cyclotomic denominator:" << endl;
                 out << HS.getCyclotomicNum();
                 out << "cyclotomic denominator:" << endl;
                 out << HS.getCyclotomicDenom();
@@ -962,7 +990,7 @@ void Output<Integer>::write_files() const {
                 // Hilbert quasi-polynomial
                 HS.computeHilbertQuasiPolynomial();
                 if (HS.isHilbertQuasiPolynomialComputed()) {
-                    out<<"Hilbert quasi-polynomial of period " << period << ":" << endl;
+                    out<<HilbertOrEhrhart+ "quasi-polynomial of period " << period << ":" << endl;
                     if(HS.get_nr_coeff_quasipol()>=0){
                         out << "only " << HS.get_nr_coeff_quasipol() << " highest coefficients computed" << endl;
                         out << "their common period is " << HS.getHilbertQuasiPolynomial().size() << "" << endl;                        
@@ -971,7 +999,7 @@ void Output<Integer>::write_files() const {
                     HQP.pretty_print(out,true);
                     out<<"with common denominator = "<<HS.getHilbertQuasiPolynomialDenom();
                 }else{
-                    out<<"Hilbert quasi-polynomial has period " << period << endl;    
+                    out<<HilbertOrEhrhart + "quasi-polynomial has period " << period << endl;    
                 }
                 out << endl << endl;
             }
@@ -984,12 +1012,18 @@ void Output<Integer>::write_files() const {
         
         if ( Result->isComputed(ConeProperty::VirtualMultiplicity)
                    && !Result->isComputed(ConeProperty::WeightedEhrhartQuasiPolynomial)) {
-            out << "virtual multiplicity of weighted Ehrhart series = "<< Result->getVirtualMultiplicity() << endl;
+            out << "virtual multiplicity = "<< Result->getVirtualMultiplicity() << endl;
+            if(Result->getVirtualMultiplicity().get_den()!=1)
+                out << "virtual multiplicity (float) = "<< std::setprecision(12) << mpq_to_nmz_float(Result->getVirtualMultiplicity()) << endl;
             out << endl;
         }
         
         if ( Result->isComputed(ConeProperty::Integral)) {
             out << "integral  = "<< Result->getIntegral() << endl;
+            if(Result->getIntegral().get_den()!=1)
+                out << "integral (float) = "<< std::setprecision(12) << mpq_to_nmz_float(Result->getIntegral()) << endl;
+            if(Result->isComputed(ConeProperty::EuclideanIntegral))
+                out << "integral (euclidean) = "<< std::setprecision(12) << Result->getEuclideanIntegral() << endl;
             out << endl;
         }
 
@@ -1039,7 +1073,7 @@ void Output<Integer>::write_files() const {
             out << endl;
         }
         if (Result->isComputed(ConeProperty::ModuleGenerators)) {
-            out << Result->getNrModuleGenerators() <<" module generators:" << endl;
+            out << Result->getNrModuleGenerators() << module_generators_name <<  ":" << endl;
             Result->getModuleGeneratorsMatrix().pretty_print(out);
             out << endl;
         }
@@ -1048,7 +1082,7 @@ void Output<Integer>::write_files() const {
             const Matrix<Integer>& Hom = Result->getDeg1ElementsMatrix();
             write_matrix_ht1(Hom);
             nr=Hom.nr_of_rows();
-            out<<nr<<" Hilbert basis elements of degree 1:"<<endl;
+            out<<nr << module_generators_name << ":"<<endl;
             Hom.pretty_print(out);
             out << endl;
         }
@@ -1157,11 +1191,14 @@ void Output<Integer>::write_files() const {
 
         //write constrains (support hyperplanes, congruences, equations)
 
-        if (Result->isComputed(ConeProperty::SupportHyperplanes)) {
+        if (Result->isComputed(ConeProperty::SupportHyperplanes) && !no_supp_hyps_output) {
             const Matrix<Integer>& Support_Hyperplanes = Result->getSupportHyperplanesMatrix();
             out << Support_Hyperplanes.nr_of_rows() <<" support hyperplanes" 
                 << of_polyhedron << ":" << endl;
-            Support_Hyperplanes.pretty_print(out);
+            if(Result->isComputed(ConeProperty::SuppHypsFloat))
+                Result->getSuppHypsFloatMatrix().pretty_print(out);
+            else
+                Support_Hyperplanes.pretty_print(out);
             out << endl;
         }
         if (Result->isComputed(ConeProperty::Sublattice)) {
@@ -1192,7 +1229,7 @@ void Output<Integer>::write_files() const {
             LatticeBasis.row_echelon_reduce();
             size_t nr_of_latt = LatticeBasis.nr_of_rows();
             if (nr_of_latt < dim ||  BasisChange.getExternalIndex()!=1) {
-                out << nr_of_latt <<" basis elements of lattice:" <<endl;
+                out << nr_of_latt <<" basis elements of generated  lattice:" <<endl;
                 LatticeBasis.pretty_print(out);
                 out << endl;
             }
