@@ -57,9 +57,12 @@ class Output {
     string of_cone;
     string of_monoid;
     string of_polyhedron;
+    string module_generators_name;
+    string HilbertOrEhrhart;
     
     bool lattice_ideal_input;
     bool no_ext_rays_output;
+    bool no_supp_hyps_output;
 
 
 //---------------------------------------------------------------------------
@@ -118,6 +121,7 @@ public:
     void set_lattice_ideal_input(bool lattice_odeal_input);
     
     void set_no_ext_rays_output();
+    void set_no_supp_hyps_output();
 
 
 //---------------------------------------------------------------------------
